/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
#ifndef _ICQ_H_
#define _ICQ_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define ICQLIBVER   0x000200
#define ICQLIBMAJOR 0
#define ICQLIBMINOR 2
#define ICQLIBMICRO 0

#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#define ICQ_LOG_OFF     0
#define ICQ_LOG_FATAL   1
#define ICQ_LOG_ERROR   2
#define ICQ_LOG_WARNING 3
#define ICQ_LOG_MESSAGE 4

#define STATUS_OFFLINE     (-1L)
#define STATUS_ONLINE      0x0000L
#define STATUS_AWAY        0x0001L
#define STATUS_DND         0x0002L /* 0x13L */
#define STATUS_NA          0x0004L /* 0x05L */
#define STATUS_OCCUPIED    0x0010L /* 0x11L */
#define STATUS_FREE_CHAT   0x0020L
#define STATUS_INVISIBLE   0x0100L
/* #define STATUS_INVISIBLE_2 0x10L */

#define ICQ_SEND_THRUSERVER       0
#define ICQ_SEND_DIRECT           1
#define ICQ_SEND_BESTWAY          2

#define ICQ_NOTIFY_SUCCESS        0
#define ICQ_NOTIFY_FAILED         1
#define ICQ_NOTIFY_CONNECTING     2
#define ICQ_NOTIFY_CONNECTED      3
#define ICQ_NOTIFY_SENT           4
#define ICQ_NOTIFY_ACK            5

#define ICQ_NOTIFY_CHAT           6
#define ICQ_NOTIFY_CHATDATA       7

#define ICQ_NOTIFY_FILECOUNT      10
#define ICQ_NOTIFY_FILEBYTES      11
#define ICQ_NOTIFY_FILESPEED      12
#define ICQ_NOTIFY_FILENICK       13
#define ICQ_NOTIFY_FILESIZE       14
#define ICQ_NOTIFY_FILENAME       15
#define ICQ_NOTIFY_FILEDATA       16
#define ICQ_NOTIFY_FILEPACKET     17
#define ICQ_NOTIFY_FILE           18

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct 
{
  const char *name;
  unsigned short code;
} COUNTRY_CODE;

typedef struct
{
  /* General */
  unsigned long icq_Uin;
  unsigned long icq_OurIP;          /* HOST byteorder */
  unsigned short icq_OurPort;       /* HOST byteorder */
  void *icq_ContactList;
  unsigned long icq_Status;
  char  *icq_Password;

  /* UDP stuff */
  int icq_UDPSok;
  unsigned char icq_UDPServMess[8192]; /* 65536 seqs max, 1 bit per seq -> 65536/8 = 8192 */
  unsigned short icq_UDPSeqNum1, icq_UDPSeqNum2;
  unsigned long icq_UDPSession;
  void *icq_UDPQueue;
  int icq_UDPExpInt;

  /* TCP stuff */
  int icq_TCPSrvSock;
  unsigned short icq_TCPSrvPort;    /* HOST byteorder */
  unsigned long icq_TCPSrvIP;       /* HOST byteorder */
  int icq_TCPSequence;
  void *icq_TCPLinks;
  void *icq_ChatSessions;
  void *icq_FileSessions;
  int TCP_maxfd;
  fd_set TCP_readfds;
  fd_set TCP_writefds;

  /* SOCKS5 Proxy stuff */
  unsigned char icq_UseProxy;
  char *icq_ProxyHost;
  unsigned long icq_ProxyIP;        /* HOST byteorder */
  unsigned short icq_ProxyPort;     /* HOST byteorder */
  int  icq_ProxyAuth;
  char *icq_ProxyName;
  char *icq_ProxyPass;
  int icq_ProxySok;
  unsigned short icq_ProxyOurPort;  /* HOST byteorder */
  unsigned long icq_ProxyDestIP;    /* HOST byteorder */
  unsigned short icq_ProxyDestPort; /* HOST byteorder */
} ICQLINK;

extern int icq_Russian;
extern unsigned char icq_LogLevel;
extern COUNTRY_CODE Country_Codes[];

/*** Callbacks ***/
extern void (*icq_Logged)(ICQLINK *link);
extern void (*icq_Disconnected)(ICQLINK *link);
extern void (*icq_RecvMessage)(ICQLINK *link, unsigned long uin,
            unsigned char hour, unsigned char minute, unsigned char day,
            unsigned char month, unsigned short year, const char *msg);
extern void (*icq_RecvURL)(ICQLINK *link, unsigned long uin, unsigned char hour,
            unsigned char minute, unsigned char day, unsigned char month,
            unsigned short year, const char *url, const char *descr);
extern void (*icq_RecvWebPager)(ICQLINK *link,unsigned char hour,
            unsigned char minute, unsigned char day, unsigned char month,
            unsigned short year, const char *nick, const char *email,
            const char *msg);
extern void (*icq_RecvMailExpress)(ICQLINK *link,unsigned char hour,
            unsigned char minute, unsigned char day, unsigned char month,
            unsigned short year, const char *nick, const char *email,
            const char *msg);
extern void (*icq_RecvChatReq)(ICQLINK *link, unsigned long uin,
            unsigned char hour, unsigned char minute, unsigned char day,
            unsigned char month, unsigned short year, const char *descr,
            unsigned long seq);
extern void (*icq_RecvFileReq)(ICQLINK *link, unsigned long uin,
            unsigned char hour, unsigned char minute, unsigned char day,
            unsigned char month, unsigned short year, const char *descr,
            const char *filename, unsigned long filesize, unsigned long seq);
extern void (*icq_RecvAdded)(ICQLINK *link, unsigned long uin,
            unsigned char hour, unsigned char minute, unsigned char day,
            unsigned char month, unsigned short year, const char *nick,
            const char *first, const char *last, const char *email);
extern void (*icq_RecvAuthReq)(ICQLINK *link, unsigned long uin,
            unsigned char hour, unsigned char minute, unsigned char day,
            unsigned char month, unsigned short year, const char *nick,
            const char *first, const char *last, const char *email,
            const char *reason);
extern void (*icq_UserFound)(ICQLINK *link, unsigned long uin, const char *nick,
            const char *first, const char *last, const char *email, char auth);
extern void (*icq_SearchDone)(ICQLINK *link);
extern void (*icq_UserOnline)(ICQLINK *link, unsigned long uin,
            unsigned long status, unsigned long ip, unsigned long port,
            unsigned long real_ip, unsigned char tcp_flag );
extern void (*icq_UserOffline)(ICQLINK *link, unsigned long uin);
extern void (*icq_UserStatusUpdate)(ICQLINK *link, unsigned long uin,
            unsigned long status);
extern void (*icq_InfoReply)(ICQLINK *link, unsigned long uin, const char *nick,
            const char *first, const char *last, const char *email, char auth);
extern void (*icq_ExtInfoReply)(ICQLINK *link, unsigned long uin,
            const char *city, unsigned short country_code, char country_stat,
            const char *state, unsigned short age, char gender,
            const char *phone, const char *hp, const char *about);
extern void (*icq_WrongPassword)(ICQLINK *link);
extern void (*icq_InvalidUIN)(ICQLINK *link);
extern void (*icq_Log)(ICQLINK *link, time_t time, unsigned char level,
            const char *str);
extern void (*icq_SrvAck)(ICQLINK *link, unsigned short seq);
extern void (*icq_RequestNotify)(ICQLINK *link, unsigned long id, int result,
            unsigned int length, void *data);
/*** Callbacks ***/

void icq_SetProxy(ICQLINK *link, const char *phost, unsigned short pport,
     int pauth, const char *pname, const char *ppass);
void icq_UnsetProxy(ICQLINK *link);

void icq_Init(ICQLINK *link, unsigned long uin, const char *password);
int icq_Connect(ICQLINK *link, const char *hostname, int port);
void icq_Disconnect(ICQLINK *link);
int icq_GetSok(ICQLINK *link);
int icq_GetProxySok(ICQLINK *link);
void icq_HandleServerResponse(ICQLINK *link);
void icq_HandleProxyResponse(ICQLINK *link);
void icq_Main(ICQLINK *link);
void icq_KeepAlive(ICQLINK *link);
void icq_Login(ICQLINK *link, unsigned long status);
void icq_Logout(ICQLINK *link);
void icq_SendContactList(ICQLINK *link);
void icq_SendVisibleList(ICQLINK *link);
void icq_SendNewUser(ICQLINK * link, unsigned long uin);
unsigned long icq_SendMessage(ICQLINK *link, unsigned long uin,
     const char *text, unsigned char thruSrv);
unsigned long icq_SendURL(ICQLINK *link, unsigned long uin, const char *url,
     const char *descr, unsigned char thruSrv);
void icq_ChangeStatus(ICQLINK *link, unsigned long status);
unsigned short icq_SendInfoReq(ICQLINK *link, unsigned long uin);
unsigned short icq_SendExtInfoReq(ICQLINK *link, unsigned long uin);
void icq_SendAuthMsg(ICQLINK *link, unsigned long uin);
void icq_SendSearchReq(ICQLINK *link, const char *email, const char *nick,
     const char* first, const char* last);
void icq_SendSearchUINReq(ICQLINK *link, unsigned long uin);

const char *icq_GetCountryName(unsigned short code);
void icq_RegNewUser(ICQLINK *link, const char *pass);
void icq_UpdateUserInfo(ICQLINK *link, const char *nick, const char *first,
     const char *last, const char *email);
void icq_UpdateAuthInfo(ICQLINK *link, unsigned long auth);
void icq_UpdateMetaInfoSet(ICQLINK *link, const char *nick, const char *first,
     const char *last, const char *email, const char *email2,
     const char *email3, const char *city, const char *state,
     const char *phone, const char *fax, const char *street,
     const char *cellular, unsigned long zip, unsigned short cnt_code,
     unsigned char cnt_stat, unsigned char emailhide);
void icq_UpdateMetaInfoHomepage(ICQLINK *link, unsigned char age,
     const char *homepage, unsigned char year, unsigned char month,
     unsigned char day, unsigned char lang1, unsigned char lang2,
     unsigned char lang3);
void icq_UpdateMetaInfoAbout(ICQLINK *link, const char *about);

void icq_FmtLog(ICQLINK *link, int level, const char *fmt, ...);

void icq_ContactAdd(ICQLINK *link, unsigned long cuin);
void icq_ContactRemove(ICQLINK *link, unsigned long cuin);
void icq_ContactClear(ICQLINK *link );
void icq_ContactSetVis(ICQLINK *link, unsigned long cuin);

/*** TCP ***/
void icq_TCPMain(ICQLINK *link);

unsigned long icq_TCPSendMessage(ICQLINK *link, unsigned long uin,
     const char *message);
unsigned long icq_TCPSendURL(ICQLINK *link, unsigned long uin,
     const char *message, const char *url);
unsigned long icq_TCPSendChatReq(ICQLINK *link, unsigned long uin,
     const char *message);
void icq_TCPAcceptChatReq(ICQLINK *link, unsigned long uin, unsigned long seq);
void icq_TCPAcceptFileReq(ICQLINK *link, unsigned long uin, unsigned long seq);

void icq_TCPSendChatData(ICQLINK *link, unsigned long uin, const char *data);
void icq_TCPCloseChat(ICQLINK *link, unsigned long uin);
void icq_TCPCloseFile(ICQLINK *link, unsigned long uin);

void icq_TCPSendFile00Packet(ICQLINK *link, unsigned long uin,
     unsigned long num_files, unsigned long total_bytes, unsigned long speed,
     const char *name);
void icq_TCPSendFile01Packet(ICQLINK *link, unsigned long uin,
     unsigned long speed, const char *name);
void icq_TCPSendFile02Packet(ICQLINK *link, unsigned long uin,
     const char *filename, unsigned long filesize, unsigned long speed);
void icq_TCPSendFile03Packet(ICQLINK *link, unsigned long uin,
     unsigned long filesize, unsigned long speed);
void icq_TCPSendFile06Packet(ICQLINK *link, unsigned long uin, int length,
     void *data);
/*** TCP ***/

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _ICQ_H_ */
